import django.dispatch

valid_confirmation_received = django.dispatch.Signal()
invalid_confirmation_received = django.dispatch.Signal()

confirmation_was_flagged = django.dispatch.Signal()
confirmation_was_approved = django.dispatch.Signal()
confirmation_was_rejected = django.dispatch.Signal()



import json

from django.http import JsonResponse, HttpResponseRedirect
from django.urls import reverse
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import View, TemplateView

from .models import PaymentConfirmation
from .utils import get_valid_keys, validate_response_code


@method_decorator(csrf_exempt, "dispatch")
class ConfirmationView(View):
    def post(self, request, *args, **kwargs):
        data = {}
        for k, v in request.POST.items():
            if k in ("x_id_factura", "x_respuesta", "x_fecha_transaccion", "x_cod_respuesta"):
                # Ignore fields in both English and Spanish
                continue
            data[k.replace("x_", "")] = v
        # TODO: Validate name consistency across all abstract models to prevent things like: id_user and customer_id.
        # Invert invoice name to keep the names consistent
        data["invoice_id"] = data["id_invoice"]
        del data["id_invoice"]

        # Get boolean fron the test_request attribute
        data["test_request"] = data["test_request"] == "TRUE"
        data["raw"] = json.dumps(request.POST)

        # Validate there are no new fields.
        # Epayco usually adds new fields without any notice.
        valid_keys = get_valid_keys()
        data = {k: v for k, v in data.items() if k in valid_keys}

        # The AbstractFlagSegment's model's save method does the flagging validations.
        item = PaymentConfirmation.objects.create(**data)
        return JsonResponse({"flag": item.is_flagged}, status=200)


@method_decorator(csrf_exempt, "dispatch")
class ResponseValidationView(TemplateView):
    template_name = "simple_payment_response.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data()
        ref = self.request.GET.get("ref_payco")
        validation = validate_response_code(ref, self.request)
        context["payment"] = validation
        return context

    def post(self, request, **kwargs):
        ref = request.POST.get("x_ref_payco", None)
        if ref is None:
            return JsonResponse({"ref": ["This field is required."]}, status=400)
        validate_response_code(ref, request)
        return HttpResponseRedirect(reverse("epayco_response_validation") + "?ref_payco={}".format(ref))



from django.contrib import admin

from .models import PaymentConfirmation


class PaymentConfirmationAdmin(admin.ModelAdmin):
    list_display = ("ref_payco", "transaction_state", "test_request", "flag", "flag_info", "created")
    list_filter = ("flag", "test_request", "transaction_state")
    search_fields = ["transaction_id", "ref_payco"]

    fieldsets = (
        (None, {"fields": ("invoice_id", "ref_payco", "description")}),
        (
            "Payment",
            {
                "classes": ("collapse",),
                "fields": ("amount", "amount_country", "amount_ok", "amount_base", "tax", "currency_code"),
            },
        ),
        ("Card", {"classes": ("collapse",), "fields": ("cardnumber", "quotas")}),
        (
            "Transaction",
            {
                "classes": ("collapse",),
                "fields": (
                    "transaction_id",
                    "transaction_state",
                    "bank_name",
                    "response",
                    "approval_code",
                    "transaction_date",
                    "cod_response",
                    "response_reason_text",
                    "errorcode",
                    "cod_transaction_state",
                    "business",
                    "franchise",
                ),
            },
        ),
        (
            "Customer",
            {
                "classes": ("collapse",),
                "fields": (
                    "cust_id_cliente",
                    "customer_doctype",
                    "customer_document",
                    "customer_name",
                    "customer_lastname",
                    "customer_email",
                    "customer_phone",
                    "customer_movil",
                    "customer_ind_pais",
                    "customer_country",
                    "customer_city",
                    "customer_address",
                    "customer_ip",
                ),
            },
        ),
        (
            "Extra",
            {
                "classes": ("collapse",),
                "fields": (
                    "extra1",
                    "extra2",
                    "extra3",
                    "extra4",
                    "extra5",
                    "extra6",
                    "extra7",
                    "extra8",
                    "extra9",
                    "extra10",
                ),
            },
        ),
        (
            "Admin",
            {
                "classes": ("collapse",),
                "fields": ("signature", "test_request", "flag", "flag_code", "flag_info", "raw"),
            },
        ),
    )


admin.site.register(PaymentConfirmation, PaymentConfirmationAdmin)



try:
    from django.urls import re_path
except:
    from django.conf.urls import url as re_path

from .views import ConfirmationView, ResponseValidationView

# app_name = 'epayco_django' # If i include this it will not find the urls.
urlpatterns = [
    re_path(
        "^epayco/confirmation$", ConfirmationView.as_view(), name="epayco_confirmation"
    ),
    re_path(
        "^payment/response-validation$",
        ResponseValidationView.as_view(),
        name="epayco_response_validation",
    ),
]



from django import template

from ..settings import epayco_settings

register = template.Library()


@register.inclusion_tag("epayco_web_checkout_button.html")
def render_checkout(
    amount,
    name,
    tax=0,
    tax_base=0,
    description="",
    currency="COP",
    country_code="co",
    external=True,
    extra1="",
    extra2="",
    extra3="",
    extra4="",
    extra5="",
    extra6="",
    extra7="",
    extra8="",
    extra9="",
    extra10="",
    same_site=True,
    request=None,
    response_url=None,
    **kwargs
):
    if description == "":
        description = name

    if epayco_settings.CONFIRMATION_URL.startswith("http"):
        confirmation_url = epayco_settings.CONFIRMATION_URL
    else:
        confirmation_url = request.build_absolute_uri(epayco_settings.CONFIRMATION_URL)

    if epayco_settings.FORCE_HTTPS and "https://" not in confirmation_url:
        confirmation_url = confirmation_url.replace("http://", "https://")

    if not response_url:
        if epayco_settings.CONFIRMATION_URL.startswith("http"):
            response_url = epayco_settings.RESPONSE_URL
        else:
            response_url = request.build_absolute_uri(epayco_settings.RESPONSE_URL)

    if epayco_settings.FORCE_HTTPS and "https://" not in response_url:
        response_url = response_url.replace("http://", "https://")

    return {
        "public_key": epayco_settings.PUBLIC_KEY,
        "amount": amount,
        "tax": tax,
        "tax_base": tax_base,
        "name": name,
        "description": description,
        "currency": currency,
        "country_code": country_code,
        "test": "{}".format(epayco_settings.TEST).lower(),
        "external": "{}".format(external).lower(),
        "extra1": extra1,
        "extra2": extra2,
        "extra3": extra3,
        "extra4": extra4,
        "extra5": extra5,
        "extra6": extra6,
        "extra7": extra7,
        "extra8": extra8,
        "extra9": extra9,
        "extra10": extra10,
        "response_url": response_url,
        "confirmation_url": confirmation_url,
        "same_site": same_site,
    }



import hashlib

import epaycosdk.epayco as Epayco
import requests

from .settings import epayco_settings


def get_signature(cust_id_client, p_key, ref_payco, transaction_id, amount, currency_code):
    """
    Genera el signature requerido por ePayco para verificar una confirmación.
    """
    signature = "{}^{}^{}^{}^{}^{}".format(cust_id_client, p_key, ref_payco, transaction_id, amount, currency_code)
    return hashlib.sha256(signature.encode("utf")).hexdigest()


def validate_response_code(ref_payco, request=None):
    """
    Verifica el ref_payco. Funciona con los 2 tipos de ref_payco.

    :param ref_payco:
    :param request:
    :return:
    """

    if not ref_payco:
        return {"valid_ref": False}

    from .models import PaymentConfirmation

    options = {
        "apiKey": epayco_settings.PUBLIC_KEY,
        "privateKey": epayco_settings.PRIVATE_KEY,
        "test": epayco_settings.TEST,
        "lenguage": "ES",
    }

    response = None
    if not ref_payco.isdigit():
        response = requests.get("https://secure.epayco.co/validation/v1/reference/{}".format(ref_payco))
        try:
            response = response.json()
        except:
            return {"valid_ref": False}
        ref_payco = str(response["data"]["x_ref_payco"])

    qs = PaymentConfirmation.objects.filter(ref_payco__iexact=ref_payco).exclude(cod_transaction_state=3)
    if not qs.exists():
        if not response:
            epayco = Epayco.Epayco(options)
            response = epayco.cash.get(ref_payco)

        # Validate the reference
        if response["success"] == True:
            if epayco_settings.CONFIRMATION_URL.startswith("http"):
                url = epayco_settings.CONFIRMATION_URL
            else:
                url = "{}://{}{}".format(
                    "https" if epayco_settings.FORCE_HTTPS or request.is_secure() else "http",
                    request.get_host(),
                    epayco_settings.CONFIRMATION_URL,
                )
            r2 = requests.post(url, data=response["data"])
            if r2.status_code == 405:
                raise Exception(
                    "There seems the be an error reaching the confirmation URL."
                    ' Please make sure you are making good use of the "FORCE_HTTPS" setting.'
                )
            obj = PaymentConfirmation.objects.filter(ref_payco__iexact=ref_payco).last()
            if obj:
                return {"valid_ref": True, "existed": False, "flag": obj.is_flagged, "obj": obj}
            else:
                return {""}
        return {"valid_ref": False}
    elif qs.filter(flag=False).exists():
        obj = qs.filter(flag=False).last()
        return {"valid_ref": True, "existed": True, "flag": False, "obj": obj}
    else:
        obj = qs.last()
        return {"valid_ref": True, "existed": True, "flag": True, "obj": obj}


def get_valid_keys():
    return [
        "cust_id_cliente",
        "ref_payco",
        "id_invoice",
        "description",
        "amount",
        "amount_country",
        "amount_ok",
        "tax",
        "amount_base",
        "currency_code",
        "bank_name",
        "cardnumber",
        "quotas",
        "response",
        "approval_code",
        "transaction_id",
        "transaction_date",
        "cod_response",
        "response_reason_text",
        "errorcode",
        "cod_transaction_state",
        "transaction_state",
        "franchise",
        "business",
        "customer_doctype",
        "customer_document",
        "customer_name",
        "customer_lastname",
        "customer_email",
        "customer_phone",
        "customer_movil",
        "customer_ind_pais",
        "customer_country",
        "customer_city",
        "customer_address",
        "customer_ip",
        "signature",
        "test_request",
        "extra1",
        "extra2",
        "extra3",
        "extra4",
        "extra5",
        "extra6",
        "extra7",
        "extra8",
        "extra9",
        "extra10",
    ]
